//****************************************************************************
// 3d.c
// (c) 1992 by Technological Computer Innovations
// Author: D. Metzler
//****************************************************************************

#include <windows.h>
#include <string.h>
#include <stdio.h>
#include "3d.h"

void PlaceCornerRivets(HWND hWnd)
{
    HDC     hDC ;
    RECT    rc ;

    hDC = GetDC(hWnd) ;
    GetClientRect(hWnd, (LPRECT) &rc) ;
    InflateRect((LPRECT) &rc, -6, -6) ;

    DrawRivet(hDC, rc.left, rc.top) ;
    DrawRivet(hDC, rc.right, rc.top) ;
    DrawRivet(hDC, rc.right, rc.bottom) ;
    DrawRivet(hDC, rc.left, rc.bottom) ;
    ReleaseDC(hWnd, hDC) ;
}

void DrawRivet(HDC hDC, int x, int y)
{
    COLORREF    color ;

    color = WHITE ;
    SetPixel(hDC, x-1, y, color) ;
    SetPixel(hDC, x-1, y-1, color) ;
    SetPixel(hDC, x, y-1, color) ;

    color = DKGRAY ;
    SetPixel(hDC, x+1, y, color) ;
    SetPixel(hDC, x+1, y+1, color) ;
    SetPixel(hDC, x, y+1, color) ;

    color = LTGRAY ;
    SetPixel(hDC, x, y, color) ;
}

void Place3DText(HDC hDC, int x, int y, LPSTR string)
{
    SetBkMode(hDC, TRANSPARENT) ;
    SetTextColor(hDC, WHITE) ;
    TextOut(hDC, x+1, y+1, (LPCSTR) string, lstrlen(string)) ;
    SetTextColor(hDC, BLUE) ;
    TextOut(hDC, x, y, (LPCSTR) string, lstrlen(string)) ;
}

void ShadowArea(HWND hWnd, HDC hDC, int ctrl_ID, HBRUSH hbrush)
{
    HWND    hCtrl ;
    HPEN    OldPen ;
    RECT    rc ;
    HBRUSH  OldBrush ;

    hCtrl = GetDlgItem(hWnd, ctrl_ID) ;
    GetClientRect(hCtrl, &rc) ;
    OldBrush = SelectObject(hDC, hbrush) ;
    OldPen = SelectObject(hDC, GetStockObject(NULL_PEN)) ;

    ClientToDialog(hWnd, hCtrl, (LPRECT) &rc) ;

    // Add shadows offset from bottom and right of client rectangle

    Rectangle(hDC, rc.left + 5, rc.bottom, rc.right + 6, rc.bottom + 6) ;
    Rectangle(hDC, rc.right, rc.top + 5, rc.right + 6, rc.bottom + 6) ;

    SelectObject (hDC, OldBrush) ;
    SelectObject (hDC, OldPen) ;
}

void Recess(HWND hWnd, HDC hDC, int ctrl_id, int depth)
{
    int     i ;
    RECT    rc ;
    HPEN    hPen, hOldPen ;
    HWND    hCtrl ;

    hCtrl = GetDlgItem(hWnd, ctrl_id) ;
    GetClientRect(hCtrl, &rc) ;
    InflateRect((LPRECT) &rc, depth, depth) ;
    ClientToDialog(hWnd, hCtrl, (LPRECT) &rc) ;

    hPen = CreatePen(PS_SOLID, 1, DKGRAY) ;

    for(i=0;i<depth;i++)
    {
        hOldPen = SelectObject(hDC, hPen) ;
        MoveTo(hDC, rc.left+i, rc.bottom-i) ;
        LineTo(hDC, rc.left+i, rc.top+i) ;
        LineTo(hDC, rc.right-i, rc.top+i) ;
        SelectObject(hDC, hOldPen) ;

        SelectObject(hDC, GetStockObject(WHITE_PEN)) ;
        LineTo(hDC, rc.right-i, rc.bottom-i) ;
        LineTo(hDC, rc.left+i, rc.bottom-i) ;
    }
    DeleteObject(hPen) ;
}

void Raise(HWND hWnd, HDC hDC, int ctrl_id, int depth)
{
    int     i ;
    RECT    rc ;
    HPEN    hPen, hOldPen ;
    HWND    hCtrl ;

    hCtrl = GetDlgItem(hWnd, ctrl_id) ;
    GetClientRect(hCtrl, &rc) ;
    InflateRect((LPRECT) &rc, depth, depth) ;
    ClientToDialog(hWnd, hCtrl, (LPRECT) &rc) ;

    hPen = CreatePen(PS_SOLID, 1, DKGRAY) ;

    for(i=0;i<depth;i++)
    {
        SelectObject(hDC, GetStockObject(WHITE_PEN)) ;
        MoveTo(hDC, rc.left+i, rc.bottom-i) ;
        LineTo(hDC, rc.left+i, rc.top+i) ;
        LineTo(hDC, rc.right-i, rc.top+i) ;

        hOldPen = SelectObject(hDC, hPen) ;
        LineTo(hDC, rc.right-i, rc.bottom-i) ;
        LineTo(hDC, rc.left+i, rc.bottom-i) ;
        SelectObject(hDC, hOldPen) ;
    }
    DeleteObject(hPen) ;
}

void MakeGroup3D(HWND hWnd, int CtrlID, int ftype)
{
    HWND    hCtrl ;
    char    buffer[35], tmp[35] ;
    RECT    rc ;
    POINT   pt ;
    int     x1, y1, x2, y2, i ;
    HDC     hDC ;
    HPEN    hPen, hOldPen ;

    hCtrl = GetDlgItem(hWnd, CtrlID) ;
    GetWindowText(hCtrl, (LPSTR) buffer, sizeof(buffer)) ;

    /* If there is a label on the groupbox, add spaces to front and back */
    if (strlen(buffer) > 0)
    {
        sprintf(tmp, " %s ", buffer) ;
        strcpy(buffer, tmp) ;
    }

    // Paint a 3-D rectangle around the control

    GetClientRect(hCtrl, (LPRECT) &rc) ;

    ClientToDialog(hWnd, hCtrl, (LPRECT) &rc) ;
    x1 = rc.left ;
    y1 = rc.top ;
    x2 = rc.right ;
    y2 = rc.bottom ;

    pt.y = y1 + 1 ;
    y1 += 7 ;

    hDC = GetDC(hWnd) ;

    rc.left = x1 ;
    rc.top = y1 ;
    rc.right = x2 ;
    rc.bottom = y2 ;
    InflateRect((LPRECT) &rc, -2, -2) ;
    for(i=0;i<6;i++)
    {
        FrameRect(hDC, (LPRECT) &rc, GetStockObject(LTGRAY_BRUSH)) ;
        InflateRect((LPRECT) &rc, +1, +1) ;
    }

    switch(ftype)
    {
        case 0:
            // Draw outside white line

            hPen = SelectObject(hDC, GetStockObject(WHITE_PEN)) ;
            MoveTo(hDC, x1-1, y2+1) ;
            LineTo(hDC, x1-1, y1-1) ;
            LineTo(hDC, x2+2, y1-1) ;

            // Draw inside white line

            MoveTo(hDC, x2-1, y1+1) ;
            LineTo(hDC, x2-1, y2-1) ;
            LineTo(hDC, x1+1, y2-1) ;

            hPen = CreatePen(PS_SOLID, 1, DKGRAY) ;
            hOldPen = SelectObject(hDC, hPen) ;

            // Draw outside dark-gray line

            MoveTo(hDC, x2+1, y1) ;
            LineTo(hDC, x2+1, y2+1) ;
            LineTo(hDC, x1-1, y2+1) ;

            // Draw inside dark-gray line

            MoveTo(hDC, x1+1, y2-1) ;
            LineTo(hDC, x1+1, y1+1) ;
            LineTo(hDC, x2-1, y1+1) ;

            SelectObject(hDC, hOldPen) ;
            DeleteObject(hPen) ;
            hPen = CreatePen(PS_SOLID, 1, LTGRAY) ;
            hOldPen = SelectObject(hDC, hPen) ;

            // Draw middle light-gray lines

            MoveTo(hDC, x1, y1) ;
            LineTo(hDC, x2, y1) ;
            LineTo(hDC, x2, y2) ;
            LineTo(hDC, x1, y2) ;
            LineTo(hDC, x1, y1) ;
            break ;

        case 1:
            rc.left = x1 ;
            rc.top = y1 ;
            rc.right = x2 ;
            rc.bottom = y2 ;
            FrameRect(hDC, (LPRECT) &rc, GetStockObject(WHITE_BRUSH)) ;

            hPen = CreatePen(PS_SOLID, 1, DKGRAY) ;
            hOldPen = SelectObject(hDC, hPen) ;

            MoveTo(hDC, x1 - 1, y2 - 1) ;
            LineTo(hDC, x1 - 1, y1 - 1) ;
            LineTo(hDC, x2, y1 - 1) ;

            MoveTo(hDC, x1 + 1, y2 - 2) ;
            LineTo(hDC, x2 - 2, y2 - 2) ;
            LineTo(hDC, x2 - 2, y1 + 1) ;
            break ;
    }

    SelectObject(hDC, hOldPen) ;
    DeleteObject(hPen) ;

    // Handle 3D text
    SetTextColor(hDC, DKGRAY) ;
    SetBkColor(hDC, LTGRAY) ;
    TextOut(hDC, x1+3, pt.y+1, (LPSTR) buffer, strlen(buffer)) ;
    SetBkMode(hDC, TRANSPARENT) ;
    SetTextColor(hDC, WHITE) ;
    TextOut(hDC, x1+1, pt.y-1, (LPSTR) buffer, strlen(buffer)) ;
    SetTextColor(hDC, BLACK) ;
    TextOut(hDC, x1+2, pt.y, (LPSTR) buffer, strlen(buffer)) ;
    ReleaseDC(hWnd, hDC) ;
}

void ClientToDialog(HWND hDlg, HWND hCtrl, LPRECT lprc)
{
    // Convert coordinates of a window control to coordinates of
    // its parent dialog window.

    // hDlg     : Handle to the dialog box
    // hCtrl    : Handle to the control
    // lprc     : ClientRect for the control: GetClientRect(hCtrl, &rc) ;

    RECT        rc ;
    POINT       pt ;

    // Convert upper-left point of the RECT
    pt.x = lprc->left ;
    pt.y = lprc->top ;
    ClientToScreen(hCtrl, (LPPOINT) &pt) ;
    ScreenToClient(hDlg, (LPPOINT) &pt) ;

    // Save the converted point in "rc"
    rc.left = pt.x ;
    rc.top = pt.y ;

    // Convert the lower-right point of the RECT
    pt.x = lprc->right ;
    pt.y = lprc->bottom ;
    ClientToScreen(hCtrl, (LPPOINT) &pt) ;
    ScreenToClient(hDlg, (LPPOINT) &pt) ;

    rc.right = pt.x ;
    rc.bottom = pt.y ;

    // Copy the new RECT to the calling RECT to return
    CopyRect(lprc, (LPRECT) &rc) ;
}

