;
; EnvRls -- Environment Release
; Written by Norm Mosher
;
; Assemble and link (Turbo Assembler):
;   tasm envrls
;   tlink /t envrls
;

bs        segment
          org  0                   ; DOS Memory-Control-Block layout
MCBtype   db   ?                   ; Type = 'M' if non-last
MCB_PID   dw   ?                   ; Program-Segment-Prefix segment addr.
MCBsize   dw   ?                   ; No. of Paragraphs this block

          org  0
PSPmark   dw   ?                   ; contains 'INT 20h' instruction
          org  2ch
PSP_ENV   dw   ?                   ; Environment segment addr.
          org  82h                 ; "Command Tail" specifies
ComTail   db   ?                   ;  name of target program

          org  256                 ; 10-byte storage area:
MyPID     dw   ?                   ; My own PSP seg,
WkMCB     dw   ?                   ;  and temporary storage
WkSeg     dw   ?                   ;   for internal use
WkSiz     dw   ?
WkPtr     dw   ?

          assume cs:bs,ds:bs,es:bs,ss:bs
          org  256
start:    mov  si,offset ComTail   ; Set addr. pointer and count
          mov  cx,9                ;  for input scan and case-convert
          mov  bx,offset WkVal+1   ; Set store-value address, too
          cld

init1:    lodsb                    ; Get input characcter
          cmp  al,' '              ; If space or lower,
          jna  init2               ;  scan is finished

          cmp  al,'a'              ; Is it lower-case alpha?
          jc   init1a              ; No: go on around
          sub  al,'a'-'A'          ; Yes: make it upper-case

init1a:   mov  [bx],al             ; Store character in Work-Value,
          inc  bx                  ;  and point to next char. slot
          loop init1               ; Check next character

error1:   mov  ax,4c01h            ; If too many characters, return
          int  21h                 ;  with ERRORLEVEL = 1

init2:    sub  si,offset ComTail-1 ; Compute no. of bytes to check
          cmp  si,3                ; If nothing between "\" and "."
          jc   error1              ;  it's no good

          mov  WkPtr,bx            ; Save RHE addr. of saved value
          mov  WkSiz,si            ; Save length of saved value

          mov  ah,52h              ; Use Dos function 52h to find
          int  21h                 ; "List-of lists" addr., and get
          mov  es,es:[bx-2]        ;   first-MCB segment addr. in ES
          mov  MyPID,cs            ; Save my own PSP segment addr.

loop1:    mov  ax,es               ; Save segment addr. of MCB, and
          mov  WkMCB,ax            ;  of next paragraph, which is 
          inc  ax                  ;   the beginning of the 
          mov  WkSeg,ax            ;    memory area itself

          cmp  ax,es:MCB_PID       ; Is this MCB for a PSP?
          jnz  loop1a              ; If not, don't bother with it;
          call chk1                ; if so, go see if it's the target
loop1a:   jz   gotcha              ; If it's the one, go release it!

          mov  ax,es:MCBsize       ; Calculate segment addr. for 
          add  ax,WkSeg            ;  next MCB

          cmp  es:MCBtype,'M'      ; See if this was non-last MCB,
          mov  es,ax               ;  and set ES to point to next one
          jz   loop1               ; If not at end, go try next MCB


;         Note -- The following four lines of code are here just in case
;              the environment to be released belongs to a program loaded
;              into upper memory by the LOADHIGH command of Version 5.0
;              of MSDOS.  This code should not cause any problems under
;              earlier DOS versions.


          cmp  ax,9fffh            ; Are we at UMB boundary (DOS 5.0)?
          jnz  errxit              ; If not, no more chances
          cmp  es:MCBtype,'M'      ; If yes, are there UMB's to do?
          jz   loop1               ; If so, go resume search

errxit:   mov  ax,4c02h            ; Target not found, so return
          int  21h                 ;  with ERRORLEVEL = 2

gotcha:   mov  WkSeg,bx            ; Save target environment segment
          dec  bx                  ; Decr. to convert to MCB segment
          mov  es,bx               ;  addr. in ES, and see if there
          cmp  es:MCBtype,'M'      ;   are any more MCBs following;
          jnz  errxit              ; if not, don't bother to release!

          mov  es,PSP_ENV          ; OK, give my own environment
          mov  ah,49h              ;  memory back to DOS now
          int  21h
          jc   errxit              ; (If no good, report failure)

          mov  es,bx               ; Now make DOS think the target
          mov  es:MCB_PID,cs       ;  environment is mine, so it will
          mov  ax,WkSeg            ;   be released when I terminate
          mov  PSP_ENV,ax
          mov  ax,4c00h            ; Return with ERRORLEVEL = 0
          int  21h                 ;  to indicate successful release!


WkVal     db   '\.........'        

chk1:     push ds
          push ax                  ; Save AX, which contains 
          mov  ds,ax               ;  Segment-adr. of PSP
          cmp  PSPmark,20cdh       ; If not really a PSP,
          jnz  return1             ;  get back right now!

          mov  bx,PSP_ENV          ; Get segment-addr. of MCB
          dec  bx                  ;  for this PSP's environment,
          mov  ds,bx               ;   and use DS to addr. MCB
          cmp  ax,MCB_PID          ; Is it this PSP's environment?
          jz   chk1a               ; If so, go check it some more!

return1:  pop  ax                  ; Restore AX contents, and
          pop  ds
          ret                      ;  get back to mainline

chk1a:    mov  cx,MCBsize          ; Convert MCB-size 
          shl  cx,1                ; (paragraph count)
          shl  cx,1                ;   to byte-count
          shl  cx,1                ;    for use as
          shl  cx,1                ;     loop-control counter

          mov  si,0                ; Set offset and
          inc  bx                  ;  segment (DS:SI) to
          mov  ds,bx               ;   scan this environment block

chk1b:    lodsb                    ; Get a character, and
          or   al,al               ;  see if it's a NULL
          loopnz Chk1b             ; If not, try next one
          jnz  return1             ; (None found, no good!)

          cmp  al,[si]             ; Is next character also NULL?
          jnz  chk1b               ; No, loop to keep trying

chk1c:    lodsb                    ; OK, now let's scan until we
          cmp  al,'.'              ;  find a '.' within this
          loopnz chk1c             ;   environment...
          jnz  return1             ; (None found, no good!)

          push es                  ; Save caller's ES, then set
          push cs                  ; ES to my own Segment addr.
          pop  es

          mov  di,cs:WkPtr         ; Use ES:DI to point to saved
          mov  cx,cs:WkSiz         ;  value, with byte-count in CX
          dec  si                  ; Back off SI to '.' we found

          std                      ; Now compare right-to-left
          repz cmpsb               ;  for intended value, and
          cld                      ;   clear direction flag

          pop  es                  ; Restore ES, then go restore
          jmp  return1             ;  AX and DS, and get back

bs        ends
          end  start
